@@ -236,20 +236,9 @@ setMethodS3("plotTracksManyChromosomes", "PairedPSCBS", function(fit, chromosome
   if (!is.null(subset)) {
     # (a) Set and unset the random seed
     if (!is.null(seed)) {
-      verbose && enter(verbose, "Setting (temporary) random seed");
-      oldRandomSeed <- NULL;
-      if (exists(".Random.seed", mode="integer")) {
-        oldRandomSeed <- get(".Random.seed", mode="integer");
-      }
-      on.exit({
-        if (!is.null(oldRandomSeed)) {
-          .Random.seed <<- oldRandomSeed;
-        }
-      }, add=TRUE);
-      verbose && cat(verbose, "The random seed will be reset to its original state afterward.");
-      verbose && cat(verbose, "Seed: ", seed);
-      set.seed(seed);
-      verbose && exit(verbose);
+      randomSeed("set", seed=seed, kind="L'Ecuyer-CMRG")
+      on.exit(randomSeed("reset"), add=TRUE)
+      verbose && printf(verbose, "Random seed temporarily set (seed=%d)\n", seed)
     }
 
     # (b) Subset
