@@ -271,22 +271,12 @@ setMethodS3("segmentByCBS", "default", function(y, chromosome=0L, x=NULL, index=
   # Set the random seed
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   if (!is.null(seed)) {
-    verbose && enter(verbose, "Setting (temporary) random seed", level=-10);
-    oldRandomSeed <- NULL;
-    if (exists(".Random.seed", mode="integer")) {
-      oldRandomSeed <- get(".Random.seed", mode="integer");
-    }
-    on.exit({
-      if (!is.null(oldRandomSeed)) {
-        .Random.seed <<- oldRandomSeed;
-      }
-    }, add=TRUE);
-    verbose && cat(verbose, "The random seed will be reset to its original state afterward.", level=-10);
-    verbose && cat(verbose, "Seed: ", seed, level=-10);
-    set.seed(seed);
-    verbose && exit(verbose);
+    randomSeed("set", seed=seed, kind="L'Ecuyer-CMRG")
+    on.exit(randomSeed("reset"), add=TRUE)
+    verbose && printf(verbose, "Random seed temporarily set (seed=%d, kind=\"L'Ecuyer-CMRG\")\n", seed)
   }
 
+
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   # Setup data
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -342,12 +332,14 @@ setMethodS3("segmentByCBS", "default", function(y, chromosome=0L, x=NULL, index=
     verbose && enter(verbose, "Segmenting multiple chromosomes");
     verbose && cat(verbose, "Number of chromosomes: ", nbrOfChromosomes);
 
-    fitList <- list();
+    fitList <- listenv()
     for (kk in seq(length=nbrOfChromosomes)) {
       chromosomeKK <- chromosomes[kk];
       chrTag <- sprintf("Chr%02d", chromosomeKK);
       verbose && enter(verbose, sprintf("Chromosome #%d ('%s') of %d", kk, chrTag, nbrOfChromosomes));
 
+      seedKK <- randomSeed("advance")
+
       # Extract subset of data and parameters for this chromosome
       dataKK <- subset(data, chrom == chromosomeKK);
       verbose && str(verbose, dataKK, level=-10);
@@ -365,44 +357,46 @@ setMethodS3("segmentByCBS", "default", function(y, chromosome=0L, x=NULL, index=
         verbose && print(verbose, knownSegmentsKK, level=-5);
       }
 
-      fit <- segmentByCBS(y=y,
-                chromosome=chrom, x=x,
-                w=w,
-                index=index,
-                undo=undo,
-                avg=avg,
-                joinSegments=joinSegments,
-                knownSegments=knownSegmentsKK,
-                ...,
-                seed=NULL,
-                verbose=verbose);
+      fitList[[chrTag]] %<=% {
+        randomSeed("set", seed=seedKK, kind="L'Ecuyer-CMRG", backup=FALSE)
 
-      # Sanity checks
-      if (R_SANITY_CHECK) {
-        if (nrow(knownSegmentsKK) == 0) {
-          # Since all missing data have been dropped...
-          stopifnot(nrow(fit$data) == length(y));
-          # ...and ordered along the genome already.
-          stopifnot(all.equal(fit$data$y, y));
-        }
+        fit <- segmentByCBS(y=y,
+                  chromosome=chrom, x=x,
+                  w=w,
+                  index=index,
+                  undo=undo,
+                  avg=avg,
+                  joinSegments=joinSegments,
+                  knownSegments=knownSegmentsKK,
+                  ...,
+                  seed=NULL,
+                  verbose=verbose);
 
-        # Assert weights were used
-        stopifnot(!hasWeights || !is.null(fit$data$w))
-      } # if (R_SANITY_CHECK)
+        # Sanity checks
+        if (R_SANITY_CHECK) {
+          if (nrow(knownSegmentsKK) == 0) {
+            # Since all missing data have been dropped...
+            stopifnot(nrow(fit$data) == length(y));
+            # ...and ordered along the genome already.
+            stopifnot(all.equal(fit$data$y, y));
+          }
 
-      rm(list=fields); # Not needed anymore
+          # Assert weights were used
+          stopifnot(!hasWeights || !is.null(fit$data$w))
+        } # if (R_SANITY_CHECK)
 
-      verbose && print(verbose, head(as.data.frame(fit)), level=-10);
-      verbose && print(verbose, tail(as.data.frame(fit)), level=-10);
+        verbose && print(verbose, head(as.data.frame(fit)), level=-10);
+        verbose && print(verbose, tail(as.data.frame(fit)), level=-10);
 
-      fitList[[chrTag]] <- fit;
+        fit
+      } ## fitList[[chrTag]] <- ...
 
-      # Not needed anymore
-      fit <- NULL;
+      rm(list=fields) # Not needed anymore
       verbose && exit(verbose);
     } # for (kk ...)
 
     verbose && enter(verbose, "Merging (independently) segmented chromosome", level=-50);
+    fitList <- as.list(fitList)
     fit <- Reduce(append, fitList);
     # Not needed anymore
     fitList <- NULL;
@@ -483,7 +477,7 @@ setMethodS3("segmentByCBS", "default", function(y, chromosome=0L, x=NULL, index=
       } # if (R_SANITY_CHECK)
     });
 
-    fitList <- list();
+    fitList <- listenv()
     for (jj in seq(length=nbrOfSegments)) {
       seg <- knownSegments[jj,];
       chromosomeJJ <- seg$chromosome;
@@ -492,77 +486,89 @@ setMethodS3("segmentByCBS", "default", function(y, chromosome=0L, x=NULL, index=
       segTag <- sprintf("chr%s:(%s,%s)", chromosomeJJ, xStart, xEnd);
       verbose && enter(verbose, sprintf("Segment #%d ('%s') of %d", jj, segTag, nbrOfSegments), level=-10);
 
+      randomSeed("advance")
+
+      ## Nothing to do?
       isSplitter <- (is.na(xStart) && is.na(xEnd));
       if (isSplitter) {
-        fit <- splitter;
-        verbose && cat(verbose, "Nothing to segment. Inserting an explicit splitter.", level=-10);
-      } else {
-        # Extract subset of data and parameters for this segment
-        dataJJ <- subset(data, chrom == chromosomeJJ & xStart <= x & x <= xEnd);
-        verbose && str(verbose, dataJJ, level=-50);
-        chrom <- x <- index <- y <- w <- NULL
-        fields <- attachLocally(dataJJ, fields=c("chrom", "x", "index", "y", "w"));
-        dataJJ <- NULL; # Not needed anymore
+        fit <- splitter
+        verbose && cat(verbose, "Nothing to segment. Inserting an explicit splitter.", level=-10)
+        fitList[[segTag]] <- fit
+        fit <- NULL
+        verbose && exit(verbose)
+        next
+      }
 
-        nbrOfLoci <- length(y);
+      # Extract subset of data and parameters for this segment
+      dataJJ <- subset(data, chrom == chromosomeJJ & xStart <= x & x <= xEnd)
+      verbose && str(verbose, dataJJ, level=-50)
+      chrom <- x <- index <- y <- w <- NULL
+      fields <- attachLocally(dataJJ, fields=c("chrom", "x", "index", "y", "w"))
+      dataJJ <- NULL # Not needed anymore
+      nbrOfLoci <- length(y)
 
-        # Empty segment?
-        # [AD HOC. Should be done by segmentCBS(). /HB 2011-10-21]
-        if(nbrOfLoci == 0) {
-          fit <- splitter;
-          fit$output$chromosome <- chromosomeJJ;
-          fit$output$start <- xStart;
-          fit$output$end <- xEnd;
-          fit$output$nbrOfLoci <- nbrOfLoci;
-        } else {
-          fit <- segmentByCBS(y=y,
-                    chromosome=chrom, x=x,
-                    w=w,
-                    index=index,
-                    undo=undo,
-                    avg=avg,
-                    joinSegments=joinSegments,
-                    knownSegments=seg,
-                    ...,
-                    seed=NULL,
-                    verbose=less(verbose,1));
-        }
+      # Empty segment?
+      # [AD HOC. Should be done by segmentCBS(). /HB 2011-10-21]
+      if(nbrOfLoci == 0) {
+        fit <- splitter
+        fit$output$chromosome <- chromosomeJJ
+        fit$output$start <- xStart
+        fit$output$end <- xEnd
+        fit$output$nbrOfLoci <- nbrOfLoci
+        fitList[[segTag]] <- fit
+        fit <- NULL
+        verbose && exit(verbose)
+        next
+      }
+
+
+      fitList[[segTag]] %<=% {
+        fit <- segmentByCBS(y=y,
+                  chromosome=chrom, x=x,
+                  w=w,
+                  index=index,
+                  undo=undo,
+                  avg=avg,
+                  joinSegments=joinSegments,
+                  knownSegments=seg,
+                  ...,
+                  seed=NULL,
+                  verbose=less(verbose,1))
 
         # Sanity checks
         if (R_SANITY_CHECK) {
-          stopifnot(nrow(fit$data) == nbrOfLoci);
-          stopifnot(all.equal(fit$data$y, y));
+          stopifnot(nrow(fit$data) == nbrOfLoci)
+          stopifnot(all.equal(fit$data$y, y))
 
           # Assert weights were used
           stopifnot(!hasWeights || !is.null(fit$data$w))
         } # if (R_SANITY_CHECK)
 
-        rm(list=fields); # Not needed anymore
+        rm(list=fields) # Not needed anymore
 
-        segs <- as.data.frame(fit);
+        segs <- as.data.frame(fit)
         if (nrow(segs) < 6) {
-          verbose && print(verbose, segs, level=-10);
+          verbose && print(verbose, segs, level=-10)
         } else {
-          verbose && print(verbose, head(segs), level=-10);
-          verbose && print(verbose, tail(segs), level=-10);
+          verbose && print(verbose, head(segs), level=-10)
+          verbose && print(verbose, tail(segs), level=-10)
         }
-      } # if (isSplitter)
 
-      # Sanity check
-      if (R_SANITY_CHECK) {
-        stopifnot(TRUE && nbrOfSegments(fit, splitters=TRUE) > 0);
-      } # if (R_SANITY_CHECK)
+        # Sanity check
+        if (R_SANITY_CHECK) {
+          stopifnot(TRUE && nbrOfSegments(fit, splitters=TRUE) > 0)
+        } # if (R_SANITY_CHECK)
 
-      fitList[[segTag]] <- fit;
+        fit
+      }
 
-      # Not needed anymore
-      fit <- NULL;
       verbose && exit(verbose);
     } # for (jj ...)
 
     verbose && enter(verbose, "Merging (independently) segmented known segments", level=-10);
-    verbose && str(verbose, fitList, level=-50)
     verbose && cat(verbose, "Number of segments: ", length(fitList), level=-10);
+    fitList <- as.list(fitList)
+    verbose && str(verbose, fitList, level=-50)
     appendT <- function(...) append(..., addSplit=FALSE);
     fit <- Reduce(appendT, fitList);
     # Not needed anymore
